@@ -18,13 +18,13 @@ struct items {
 
 struct CheckboxTree {
     newtComponent sb;
-    int curWidth;	/* size of text w/o scrollbar or border*/
-    int curHeight;	/* size of text w/o border */
     struct items * itemlist;
     struct items ** flatList, ** currItem, ** firstItem;
     int flatCount;
     int flags;
-    int pad;
+    int sbAdjust;
+    int curWidth;
+    int userHasSetWidth;
     char * seq;
     char * result;
 };
@@ -39,6 +39,8 @@ static void buildFlatList(newtComponent co);
 static void doBuildFlatList(struct CheckboxTree * ct, struct items * item);
 enum countWhat { COUNT_EXPOSED=0, COUNT_SELECTED=1 };
 static int countItems(struct items * item, enum countWhat justExposed);
+static inline void updateWidth(newtComponent co, struct CheckboxTree * ct,
+				int maxField);
 
 static struct componentOps ctOps = {
     ctDraw,
@@ -48,6 +50,15 @@ static struct componentOps ctOps = {
     ctMapped,
 } ;
 
+static inline void updateWidth(newtComponent co, struct CheckboxTree * ct,
+				int maxField) {
+    ct->curWidth = maxField;
+    co->width = ct->curWidth + ct->sbAdjust;
+
+    if (ct->sb)
+	ct->sb->left = co->left + co->width - 1;
+}
+
 static int countItems(struct items * item, enum countWhat what) {
     int count = 0;
 
@@ -158,7 +169,7 @@ int newtCheckboxTreeAddArray(newtComponent co,
 			    int flags, int * indexes) {
     struct items * curList, * newNode, * item;
     struct items ** listPtr = NULL;
-    int i, index, numIndexes;
+    int i, index, numIndexes, width;
     struct CheckboxTree * ct = co->data;
 
     numIndexes = 0;
@@ -232,9 +243,10 @@ int newtCheckboxTreeAddArray(newtComponent co,
     item->depth = numIndexes - 1;
 
     i = 4 + (3 * item->depth);
+    width = wstrlen(text, -1);
 
-    if ((strlen(text) + i + ct->pad) > co->width) {
-	co->width = strlen(text) + i + ct->pad;
+    if ((ct->userHasSetWidth == 0) && ((width + i + ct->sbAdjust) > co->width)) {
+	updateWidth(co, ct, width + i);
     }
 
     return 0;
@@ -266,6 +278,16 @@ static void listSelected(struct items * items, int * num, const void ** list, in
     }
 }
 
+void newtCheckboxTreeSetWidth(newtComponent co, int width) {
+    struct CheckboxTree * ct = co->data;
+
+    co->width = width;
+    ct->curWidth = co->width - ct->sbAdjust;
+    ct->userHasSetWidth = 1;
+    if (ct->sb) ct->sb->left = co->width + co->left - 1;
+    ctDraw(co);
+}
+
 const void ** newtCheckboxTreeGetSelection(newtComponent co, int *numitems)
 {
     return newtCheckboxTreeGetMultiSelection(co, numitems, 0);
@@ -314,21 +336,26 @@ newtComponent newtCheckboxTreeMulti(int left, int top, int height, char *seq, in
     co->height = height;
     co->width = 0;
     co->isMapped = 0;
+    ct->curWidth = 0;
+    ct->userHasSetWidth = 0;
     ct->itemlist = NULL;
     ct->firstItem = NULL;
     ct->currItem = NULL;
     ct->flatList = NULL;
-	if (seq)
-	  ct->seq = strdup(seq);
-	else
-	  ct->seq = strdup(" *");
+
+    ct->flags = flags;
+
+    if (seq)
+	ct->seq = strdup(seq);
+    else
+	ct->seq = strdup(" *");
     if (flags & NEWT_FLAG_SCROLL) {
 	ct->sb = newtVerticalScrollbar(left, top, height,
 				       COLORSET_LISTBOX, COLORSET_ACTLISTBOX);
-	ct->pad = 2;
+	ct->sbAdjust = 2;
     } else {
 	ct->sb = NULL;
-	ct->pad = 0;
+	ct->sbAdjust = 0;
     }
     
     return co;
@@ -371,7 +398,7 @@ int ctSetItem(newtComponent co, struct items *item, enum newtFlagsSense sense)
 	case NEWT_FLAGS_TOGGLE:
 	    if (item->branch)
 	      item->selected = !item->selected;
-	    else {
+	    else if (!(ct->flags & NEWT_CHECKBOXTREE_UNSELECTABLE)) {
 		    item->selected++;
 		    if (item->selected==strlen(ct->seq))
 		      item->selected = 0;
@@ -426,6 +453,9 @@ static void ctDraw(newtComponent co) {
     item = ct->firstItem;
     
     i = 0;
+
+    newtTrashScreen();
+    
     while (*item && i < co->height) {
 	newtGotorc(co->top + i, co->left);
 	if (*item == *ct->currItem) {
@@ -443,13 +473,22 @@ static void ctDraw(newtComponent co) {
 	    else
 		SLsmg_write_string("<+> ");
 	} else {
-	    char tmp[5];
-	    snprintf(tmp,5,"[%c] ",ct->seq[(*item)->selected]);
-	    SLsmg_write_string(tmp);
+	    if (ct->flags & NEWT_CHECKBOXTREE_HIDE_BOX) {
+		if ((*item)->selected)
+		    SLsmg_set_color(NEWT_COLORSET_ACTLISTBOX);
+	        SLsmg_write_string("    ");
+	    } else {
+	        char tmp[5];
+	        snprintf(tmp,5,"[%c] ",ct->seq[(*item)->selected]);
+	        SLsmg_write_string(tmp);
+	    }
 	}
 
 	SLsmg_write_nstring((*item)->text, co->width - 4 - 
 					   (3 * (*item)->depth));
+
+	SLsmg_set_color(NEWT_COLORSET_LISTBOX);
+
 	item++;
 	i++;
     }
@@ -657,7 +696,7 @@ void newtCheckboxTreeSetEntry(newtComponent co, const void * data, const char *
 {
     struct CheckboxTree * ct;
     struct items * item;
-    int i;
+    int i, width;
 
     if (!co) return;
     ct = co->data;
@@ -669,8 +708,9 @@ void newtCheckboxTreeSetEntry(newtComponent co, const void * data, const char *
 
     i = 4 + (3 * item->depth);
 
-    if ((strlen(text) + i + ct->pad) > co->width) {
-	co->width = strlen(text) + i + ct->pad;
+    width = wstrlen(text, -1);
+    if ((ct->userHasSetWidth == 0) && ((width + i + ct->sbAdjust) > co->width)) {
+	updateWidth(co, ct, width + i);
     }
 
     ctDraw(co);
@@ -712,3 +752,43 @@ void newtCheckboxTreeSetEntryValue(newtComponent co, const void * data, char val
     ctDraw(co);
 }
 
+
+void newtCheckboxTreeSetCurrent(newtComponent co, void * data) {
+    struct CheckboxTree * ct = co->data;
+    int * path;
+    int i, j;
+    struct items * treeTop, * item;
+
+    path = newtCheckboxTreeFindItem(co, data);
+    if (!path) return;
+
+    /* traverse the path and turn on all of the branches to this point */
+    for (i = 0, treeTop = ct->itemlist; path[i + 1] != NEWT_ARG_LAST; i++) {
+	for (j = 0, item = treeTop; j < path[i]; j++)
+	    item = item->next;
+
+	item->selected = 1;
+	treeTop = item->branch;
+    }
+
+    buildFlatList(co);
+	
+    item = findItem(ct->itemlist, data);
+
+    i = 0;
+    while (ct->flatList[i] != item) i++;
+
+    /* choose the top item */
+    j = i - (co->height / 2);
+
+    if ((j + co->height) > ct->flatCount) 
+	j = ct->flatCount - co->height;
+    
+    if (j < 0)
+	j = 0;
+
+    ct->firstItem = ct->flatList + j;
+    ct->currItem = ct->flatList + i;
+
+    ctDraw(co);
+}
